#
# Lightweight Zope startup
#
# Fast Zope startup is achieved by not installing (m)any
# products. If your tests require a product you must
# install it yourself using installProduct().
#
# Typically used as in
#
#   import ZopeLite as Zope
#   Zope.installProduct('SomeProduct')
#   app = Zope.app()
#

# $Id: ZopeLite.py 7739 2005-03-26 15:55:51Z shh42 $

import os, sys, time

# Allow code to tell it is run by the test framework
os.environ['ZOPETESTCASE'] = '1'

# Increase performance on MP hardware
sys.setcheckinterval(2500)

# Shut up if we are not in control of the import process
_quiet = sys.modules.has_key('Zope')

def _print(msg):
    '''Writes 'msg' to stderr and flushes the stream.'''
    sys.stderr.write(msg)
    sys.stderr.flush()

def _write(msg):
    '''Writes 'msg' to stderr if not _quiet.'''
    if not _quiet:
        _print(msg)

def _exec(cmd):
    '''Prints the time it takes to execute 'cmd'.'''
    if os.environ.get('X', None):
        start = time.time()
        exec cmd
        _print('(%.3fs)' % (time.time() - start))

_write('Loading Zope, please stand by ')
_start = time.time()

def _configure_logging():
    # Initialize the logging module
    if not sys.modules.has_key('logging'):
        import logging
        logging.basicConfig()

def _configure_debug_mode():
    # Switch off debug mode
    import App.config
    config = App.config.getConfiguration()
    config.debug_mode = 0
    App.config.setConfiguration(config)

def _configure_client_cache():
    # Make sure we use a temporary client cache
    import App.config
    config = App.config.getConfiguration()
    config.zeo_client_name = None
    App.config.setConfiguration(config)

try:
    import App.config
except ImportError:
    pass # Zope < 2.7
else:
    _configure_logging()
    _configure_debug_mode()
    _configure_client_cache()
    # Need to import Zope early on as the ZTUtils
    # package of Zope < 2.7.4 relies on it.
    _exec('import Zope')
    import Zope

_exec('import ZODB')
import ZODB
_write('.')

_exec('import Globals')
import Globals
# Work around a bug in Zope 2.7.0
Globals.DevelopmentMode = 0
_exec('import OFS.SimpleItem')
import OFS.SimpleItem
_exec('import OFS.ObjectManager')
import OFS.ObjectManager
_write('.')

_exec('import OFS.Application')
import OFS.Application
import App.ProductContext

def _apply_patches():
    # Avoid expensive product import
    def null_import_products(): pass
    OFS.Application.import_products = null_import_products

    # Avoid expensive product installation
    def null_initialize(app): pass
    OFS.Application.initialize = null_initialize

    # Avoid expensive help registration
    def null_register_topic(self,id,topic): pass
    App.ProductContext.ProductContext.registerHelpTopic = null_register_topic
    def null_register_title(self,title): pass
    App.ProductContext.ProductContext.registerHelpTitle = null_register_title
    def null_register_help(self,directory='',clear=1,title_re=None): pass
    App.ProductContext.ProductContext.registerHelp = null_register_help

try:
    import App.config
except ImportError:
    # Zope < 2.7
    _apply_patches()
    # Make sure we use a temporary client cache
    if os.environ.get('ZEO_CLIENT'):
        del os.environ['ZEO_CLIENT']
    # Load Zope
    _exec('import Zope')
    import Zope
else:
    # Do not patch a running Zope
    if not getattr(Zope, '_began_startup', 0):
        _apply_patches()
_write('.')

# Allow test authors to install Zope products into the test environment. Note
# that installProduct() must be called at module level -- never from tests.
from OFS.Application import get_folder_permissions, get_products, install_product
from OFS.Folder import Folder
import Products

_theApp = Zope.app()
_installedProducts = {}

def hasProduct(name):
    '''Checks if a product can be found along Products.__path__'''
    return name in [n[1] for n in get_products()]

def installProduct(name, quiet=0):
    '''Installs a Zope product.'''
    start = time.time()
    meta_types = []
    if not _installedProducts.has_key(name):
        for priority, product_name, index, product_dir in get_products():
            if product_name == name:
                if not quiet: _print('Installing %s ... ' % product_name)
                # We want to fail immediately if a product throws an exception
                # during install, so we set the raise_exc flag.
                install_product(_theApp, product_dir, product_name, meta_types,
                                get_folder_permissions(), raise_exc=1)
                _installedProducts[product_name] = 1
                Products.meta_types = Products.meta_types + tuple(meta_types)
                Globals.default__class_init__(Folder)
                if not quiet: _print('done (%.3fs)\n' % (time.time() - start))
                break
        else:
            if name != 'SomeProduct':   # Ignore the skeleton tests :-P
                if not quiet: _print('Installing %s ... NOT FOUND\n' % name)

def _load_control_panel():
    # Loading the Control_Panel of an existing ZODB may take
    # a while; print another dot if it does.
    start = time.time()
    max = (start - _start) / 4
    _exec('_theApp.Control_Panel')
    cp = _theApp.Control_Panel
    if hasattr(cp, 'initialize_cache'):
        cp.initialize_cache()
    if (time.time() - start) > max:
        _write('.')

def _install_products():
    installProduct('PluginIndexes', 1)  # Must install first
    installProduct('OFSP', 1)
    #installProduct('ExternalMethod', 1)
    #installProduct('ZSQLMethods', 1)
    #installProduct('ZGadflyDA', 1)
    #installProduct('MIMETools', 1)
    #installProduct('MailHost', 1)

_load_control_panel()
_install_products()

# So people can use ZopeLite.app()
app = Zope.app
debug = Zope.debug
DB = Zope.DB
# startup appeared in Zope 2.6.1
def startup(): pass
# configure appeared in Zope 2.7
try: configure = Zope.configure
except AttributeError: pass

# ZODB sandbox factory
from ZODB.DemoStorage import DemoStorage

def sandbox(base=None):
    '''Returns a sandbox copy of the base ZODB.'''
    if base is None: base = Zope.DB
    base_storage = base._storage
    quota = getattr(base_storage, '_quota', None)
    storage = DemoStorage(base=base_storage, quota=quota)
    return ZODB.DB(storage)

_write(' done (%.3fs)\n' % (time.time() - _start))

